<?php declare(strict_types=1);

namespace App\Repository;

use App\Entity\MenuItem;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bridge\Doctrine\Types\UlidType;
use Symfony\Component\Uid\Ulid;

/**
 * @extends ServiceEntityRepository<MenuItem>
 */
final class MenuItemRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, MenuItem::class);
    }

    public function getSuitableAliasQueryBuilder(MenuItem $menuItem): \Doctrine\ORM\QueryBuilder
    {
        $ids = $this->createQueryBuilder('m')
            ->select('DISTINCT IDENTITY(m.aliasOf)')
            ->where('m.deletedAt IS NULL')
            ->andWhere('m.aliasOf IS NOT NULL')
            ->getquery();
        $ids = $ids->getScalarResult();
        $ids = array_map(static fn(array $id): Ulid => Ulid::fromBinary($id[1]), $ids);

        $qb = $this->createQueryBuilder('m');
        $qb
            ->where('m.foodVendor = :vendorId')
            ->andWhere('m.deletedAt IS NULL')
            ->andWhere('m.id != :id');
        foreach ($ids as $key => $id) {
            $qb->andWhere("m.id != :idBy{$key}")
                ->setParameter("idBy{$key}", $id, UlidType::NAME);
        }
        $qb
            ->orderBy('m.name', 'ASC')
            ->setParameter('vendorId', $menuItem->getFoodVendor()->getId(), UlidType::NAME)
            ->setParameter('id', $menuItem->getId(), UlidType::NAME);
        return $qb;
    }
}
